@@ -41,6 +41,10 @@
 #include <string>
 #include <new>
 
+#ifndef HAVE_HEIMDAL_VERSION
+#include "gssapi_headerfix.h"
+#endif
+
 using namespace gss_eap_util;
 
 struct gss_eap_attr_provider;
@@ -349,7 +353,7 @@ gssEapSetNameAttribute(OM_uint32 *minor,
 
 OM_uint32
 gssEapExportAttrContext(OM_uint32 *minor,
-                        gss_name_t name,
+                        gss_const_name_t name,
                         gss_buffer_t buffer);
 
 OM_uint32
@@ -359,7 +363,7 @@ gssEapImportAttrContext(OM_uint32 *minor,
 
 OM_uint32
 gssEapDuplicateAttrContext(OM_uint32 *minor,
-                           gss_name_t in,
+                           gss_const_name_t in,
                            gss_name_t out);
 
 OM_uint32
